mod emitter;
mod parser;
mod transformer;

use anyhow::Result;
use std::{env, fs};
use thiserror::Error;

#[derive(Error, Debug)]
pub enum ArgumentError {
    #[error("Usage: {0} ./path/to/file.solace")]
    MissingSourceFile(String),
}

fn main() -> Result<()> {
    let args: Vec<String> = env::args().collect();

    if args.len() < 2 {
        return Err(ArgumentError::MissingSourceFile(args[0].clone()).into());
    }

    let file_path = &args[1];
    let input = fs::read_to_string(file_path).expect(&format!("Cannot read file '{}'!", file_path));

    // Parse Solace Code
    let ast = parser::parse(&input, /* debug */ true)?;

    // Transform from Solace AST to SWC AST
    let js_transformer = transformer::js::JsTransformer::new();
    let js_ast = js_transformer.transform(ast);

    // Emit JavaScript
    let js_emitter = emitter::js::JsEmitter::new();
    let js_code = js_emitter.emit(js_ast)?;

    // Write Output to stdout
    println!("{}", js_code);

    Ok(())
}
